package sdl

/* 

#cgo pkg-config: sdl
#include <SDL.h>
#include "defs.h"

*/
import "C"
import "unsafe"
import "math"

func Init(flags uint32) int {
	return int(C.SDL_Init(C.Uint32(flags)))
}

func Quit() {
	C.SDL_Quit()
}

func SetVideoMode(w, h, bpp int, flags uint32) *Surface {
	return (*Surface)(unsafe.Pointer((C.SDL_SetVideoMode(C.int(w), C.int(h), C.int(bpp), C.Uint32(flags)))))
}

//export goAudioCallback
func goAudioCallback(handle unsafe.Pointer, stream *byte, length int) {
	//b := C.GoBytes(unsafe.Pointer(stream), C.int(length))
	//h := (*AudioHandle)(handle)
	//h.Callback(h.Userdata, b)
}

func newCAudioSpec(s *AudioSpec) C.SDL_AudioSpec {
	return C.SDL_AudioSpec{
		freq:     C.int(s.Freq),
		format:   C.Uint16(s.Format),
		channels: C.Uint8(s.Channels),
		samples:  C.Uint16(s.Samples),
	}
}

func OpenAudio(desired, obtained *AudioSpec) int {
	d := newCAudioSpec(desired)
	o := C.SDL_AudioSpec{}
	h := (*C.audio_handle_t)(unsafe.Pointer(&desired.AudioHandle))

	C.set_audio_callback(&d, h)
	return int(C.SDL_OpenAudio(&d, &o))
}

func PauseAudio(pause_on int) {
	C.SDL_PauseAudio(C.int(pause_on))
}

func GL_SetAttribute(attr, value int) int {
	return int(C.SDL_GL_SetAttribute(C.SDL_GLattr(attr), C.int(value)))
}

func GL_SwapBuffers() {
	C.SDL_GL_SwapBuffers()
}

func PumpEvents() {
	C.SDL_PumpEvents()
}

func PollEvent() (ev Event) {
	var cev C.SDL_Event
	if C.SDL_PollEvent(&cev) == 0 {
		return
	}

	typ := *((*uint8)(unsafe.Pointer(&cev)))
	switch typ {
	case QUIT:
		ev = *((*QuitEvent)(unsafe.Pointer(&cev)))
	case ACTIVEEVENT:
		ev = *((*ActiveEvent)(unsafe.Pointer(&cev)))
	case KEYDOWN, KEYUP:
		ev = *((*KeyboardEvent)(unsafe.Pointer(&cev)))
	case MOUSEMOTION:
		ev = *((*MouseMotionEvent)(unsafe.Pointer(&cev)))
	case MOUSEBUTTONDOWN, MOUSEBUTTONUP:
		ev = *((*MouseButtonEvent)(unsafe.Pointer(&cev)))
	case JOYAXISMOTION:
		ev = *((*JoyAxisEvent)(unsafe.Pointer(&cev)))
	case JOYBALLMOTION:
		ev = *((*JoyBallEvent)(unsafe.Pointer(&cev)))
	case JOYHATMOTION:
		ev = *((*JoyBallEvent)(unsafe.Pointer(&cev)))
	case SYSWMEVENT:
		ev = *((*SysWMEvent)(unsafe.Pointer(&cev)))
	case VIDEORESIZE:
		ev = *((*ResizeEvent)(unsafe.Pointer(&cev)))
	case VIDEOEXPOSE:
		ev = *((*ExposeEvent)(unsafe.Pointer(&cev)))
	case USEREVENT:
		ev = *((*UserEvent)(unsafe.Pointer(&cev)))

	}

	return
}

func Delay(ms uint) {
	C.SDL_Delay(C.Uint32(ms))
}

func GetError() string {
	return C.GoString(C.SDL_GetError())
}

func ClearError() {
	C.SDL_ClearError()
}

func WM_SetCaption(title, icon string) {
	C.SDL_WM_SetCaption(C.CString(title), C.CString(icon))
}

func ShowCursor(toggle int) int {
	return int(C.SDL_ShowCursor(C.int(toggle)))
}

func Flip(screen *Surface) {
	C.SDL_Flip((*C.SDL_Surface)(unsafe.Pointer(screen)))
}

func LockSurface(surface *Surface) int {
	return int(C.SDL_LockSurface((*C.SDL_Surface)(unsafe.Pointer(surface))))
}

func UnlockSurface(surface *Surface) {
	C.SDL_UnlockSurface((*C.SDL_Surface)(unsafe.Pointer(surface)))
}

func BlitSurface(src *Surface, srcrect *Rect, dst *Surface, dstrect *Rect) int {
	return int(C.SDL_BlitSurface((*C.SDL_Surface)(unsafe.Pointer(src)),
		(*C.SDL_Rect)(unsafe.Pointer(srcrect)),
		(*C.SDL_Surface)(unsafe.Pointer(dst)),
		(*C.SDL_Rect)(unsafe.Pointer(dstrect))))
}

func MUSTLOCK(surface *Surface) bool {
	if surface.Offset != 0 || surface.Flags&(HWSURFACE|ASYNCBLIT|RLEACCEL) != 0 {
		return true
	}

	return false
}

func MapRGB(fmt *PixelFormat, r, g, b uint8) uint32 {
	return uint32(C.SDL_MapRGB((*C.SDL_PixelFormat)(unsafe.Pointer(fmt)), C.Uint8(r), C.Uint8(g), C.Uint8(b)))
}

func UpdateRect(screen *Surface, x, y, w, h int32) {
	C.SDL_UpdateRect((*C.SDL_Surface)(unsafe.Pointer(screen)), C.Sint32(x), C.Sint32(y), C.Uint32(w), C.Uint32(h))
}

func CreateRGBSurfaceFrom(pixels []byte, width, height, depth, pitch int, Rmask, Gmask, Bmask, Amask uint32) *Surface {
	return (*Surface)(unsafe.Pointer((C.SDL_CreateRGBSurfaceFrom((unsafe.Pointer(&pixels[0])), (C.int)(width),
		(C.int)(height), (C.int)(depth), (C.int)(pitch), (C.Uint32)(Rmask),
		(C.Uint32)(Gmask), (C.Uint32)(Bmask), C.Uint32(Amask)))))
}

func InitSubSystem(flags uint32) int {
	return (int)(C.SDL_InitSubSystem(C.Uint32(flags)))
}

func QuitSubSystem(flags uint32) {
	C.SDL_QuitSubSystem(C.Uint32(flags))
}

func GetKeyState(keys []uint8) []uint8 {
	var size C.int

	ptr := C.SDL_GetKeyState(&size)
	numkeys := int(size)
	if int(numkeys) > len(keys) {
		keys = make([]uint8, numkeys)
	}

	dptr := (*[math.MaxInt32]uint8)(unsafe.Pointer(ptr))

	for i := 0; i < numkeys; i++ {
		keys[i] = (*dptr)[i]
	}

	return keys
}

func BlitBuffer(src *Surface, w, h, xoff, yoff int, pix []uint32) {
	if MUSTLOCK(src) {
		if LockSurface(src) < 0 {
			return
		}
	}

	switch src.Format.BytesPerPixel {
	case 4: // 32 bpp
		for i, y := 0, 0; y < h; y++ {
			for x := 0; x < w; x++ {
				var bufp *[math.MaxInt32]C.Uint32
				bufp = (*[math.MaxInt32]C.Uint32)(unsafe.Pointer(src.Pixels))

				k := (y+yoff)*int(src.Pitch)/4 + (x + xoff)
				c := MapRGB(src.Format, uint8(pix[i]&0xFF), uint8((pix[i]>>8)&0xFF), uint8((pix[i]>>16)&0xFF))
				bufp[k] = C.Uint32(c)
				i++
			}
		}

	default:
		panic("not implemented")
	}

	if MUSTLOCK(src) {
		UnlockSurface(src)
	}
}
